#ifndef XUNIX_H
#define XUNIX_H

#include <unistd.h>
#include <csignal>
#include <stdexcept>
#include <string>

namespace xunix {

  // xunix::exception converts unix errors associated with errno into
  // C++ exceptions.
  class exception : public std::exception{
  public:
    exception(const std::string& prefix);
    virtual ~exception() throw();
    virtual const char* what() const throw();
    virtual int error_number() const throw();
  private:
    void init(const std::string& prefix);
    std::string _what;
    // "errno" is sometimes a macro.  Use something else.
    int _error_number;
  };

  //
  // Signals
  //

  void sigaction(int signum,
                 const struct sigaction* act,
                 struct sigaction* oldact) throw(xunix::exception);

  void sigemptyset(sigset_t* sigset);
  void sigfillset(sigset_t* sigset);
  void sigaddset(sigset_t* sigset, int signum);
  void sigdelset(sigset_t* sigset, int signum);
   int sigismember(const sigset_t* sigset, int signum);

   void pause(void);

   //
   // Pipes
   //

   void pipe(int filedes[2]);

   //
   // Select
   //
   int select(int n,
              fd_set* rset,
              fd_set* wset,
              fd_set* eset,
              struct timeval* t);

   //
   // File Operations
   //
   ssize_t read(int fd, void *buf, size_t count);
   ssize_t write(int fd, const void *buf, size_t count); 

}

#endif  /*  XUNIX_H  */
